@@ -1,6 +1,6 @@
 # perldoc RRSIG.pm for documentation.
 # Specs: RFC 2535 section 4
-# $Id: RRSIG.pm 318 2005-05-30 16:36:52Z olaf $
+# $Id: RRSIG.pm 1105 2013-08-23 09:03:10Z willem $
 
 package Net::DNS::RR::RRSIG;
 
@@ -9,17 +9,19 @@ use vars qw(@ISA $VERSION @EXPORT );
 use Net::DNS;
 use Carp;
 use bytes;
+use Data::Dumper;
 
 use Crypt::OpenSSL::DSA;
 use Crypt::OpenSSL::RSA;
 use Crypt::OpenSSL::Bignum;
 use Net::DNS::SEC::Private;
 
+
 use File::Basename;
 use MIME::Base64;
 use Math::BigInt;
 use Time::Local;
-use Digest::SHA1 qw (sha1);
+use Digest::SHA qw (sha1 sha256 sha512);
 
 
 
@@ -33,22 +35,24 @@ use Digest::SHA1 qw (sha1);
 
 require Exporter;
 
-$VERSION = do { my @r=(q$Revision: 318 $=~/\d+/g); sprintf "%d."."%03d"x$#r,@r };
+$VERSION = do { my @r=(q$Revision: 1105 $=~/\d+/g); sprintf "%d."."%03d"x$#r,@r };
 @ISA = qw (
 	   Exporter
   	 Net::DNS::RR
   	 Net::DNS::SEC
 	   );
 
-@EXPORT = qw (
+
+@EXPORT = qw (          
 	      );
 
 
-use strict;
-my $debug=0;
-my $crypt_open_ssl=1;
 
 
+use strict;
+my $crypt_open_ssl=1;
+my $debug=0;
+
 
 
 
@@ -82,7 +86,7 @@ sub new {
 	$self->{"keytag"}=unpack("n",substr($$data,$offsettokeytag,2));
 	my($signame,$sigoffset) = Net::DNS::Packet::dn_expand
 	    ($data, $offsettosignm);
-	$self->{"signame"}=lc($signame).".";  #Add a trailing dot..dn_expand does not do that.
+	$self->{"signame"}=lc($signame); 
 	my($sigmaterial)=substr($$data,$sigoffset,
 				($self->{"rdlength"}-$sigoffset+$offset));
 	$self->{"sigbin"}=$sigmaterial;
@@ -106,7 +110,7 @@ sub new_from_string {
 	    $labels, $orgttl, $sigexpiration,
 	    $siginception, $keytag,$signame,$sig) = 
 		$string =~ 
-		    /^\s*(\S+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\S+)\s+(.*)/;
+		    /^\s*(\S+)\s+(\S+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\S+)\s+(.*)/;
 	croak (" Invallid RRSIG RR, check your fomat ") if !$keytag;
 	$sig =~ s/\s*//g;
 	$self->{"typecovered"}= $typecovered;
@@ -118,7 +122,7 @@ sub new_from_string {
 	$self->{"sigexpiration"}=  $sigexpiration;
 	$self->{"siginception"}= $siginception;
 	$self->{"keytag"}= $keytag;
-	$self->{"signame"}= lc($signame);
+	$self->{"signame"}= lc(Net::DNS::stripdot($signame));
 	$self->{"sig"}= $sig;
 	$self->{"sigbin"}= decode_base64($sig);
 	$self->{"vrfyerrstr"}="";
@@ -138,7 +142,7 @@ sub rdatastr {
 	    $rdatastr .= "  "  . "$self->{sigexpiration}";
 	    $rdatastr .= " (\n\t\t\t"  . "$self->{siginception}";
 	    $rdatastr .= " "  . "$self->{keytag}";
-	    $rdatastr .= "  "  . "$self->{signame}";
+	    $rdatastr .= "  "  . "$self->{signame}.";
 	    # do some nice formatting
 	    my $sigstring=$self->{sig};
 	    $sigstring =~ s/\n//g;
@@ -172,7 +176,7 @@ sub rr_rdata_without_sigbin {
 	$rdata .= pack("n",$self->{"keytag"});
 	# Since we will need canonical and expanded names while checking 
 	# we do not use the packet->dn_comp here but use RFC1035 p10.
-	{   my @dname= split /\./,lc($self->{"signame"});
+	{   my @dname= split /\./,lc($self->{"signame"}.".");  #/ emacs fontlock
 	    for (my $i=0;$i<@dname;$i++){
 		$rdata .= pack ("C",length $dname[$i] );
 		$rdata .= $dname[$i] ;
@@ -193,6 +197,7 @@ sub rr_rdata {
 	$rdata=$self->rr_rdata_without_sigbin;
 	
 	if ($self->{"sig"} ne "NOTYETCALCULATED") {
+            $self->{"sigbin"}= decode_base64($self->{"sig"}) unless defined $self->{"sigbin"} ;
 	    $rdata .= $self->{"sigbin"};
 	}else{
 	    die "RRSIGs should not be used for SIG0 type signatures, use Net::DNS::RR::SIG";
@@ -220,12 +225,15 @@ sub create {
 	$Private=Net::DNS::SEC::Private->new($priv_key);
     }
 
-    die "Create did not manage obtain a Net::DNS::SEC::Private object "unless (UNIVERSAL::isa($Private,"Net::DNS::SEC::Private"));
-
+    unless (UNIVERSAL::isa($Private,"Net::DNS::SEC::Private")){
+	$self->{"sigerrstr"}= "Create did not manage to parse a private key into a Net::DNS::SEC::Private object ";
+	return (0);
+	    
+    }
     $self->{"algorithm"}=Net::DNS::SEC->algorithm($Private->algorithm);
     $self->{"keytag"}=$Private->keytag;
-    $self->{"signame"}=$Private->signame;
-
+    $self->{"signame"}=Net::DNS::stripdot($Private->signame);
+   
 
     die "Argument is not a reference to an array, are you trying to create a SIG0 using RRSIG?" if ! ref ($datarrset);
 
@@ -242,10 +250,10 @@ sub create {
 
 
     if (defined ($args{ttl})){
-	print "\nSetting TTL to ".  $args{"ttl"} if $debug;
+	print "Setting TTL to ".  $args{"ttl"} . "\n" if $debug;
 	$self->{"ttl"}= $args{"ttl"};
     }else{
-	$self->{"ttl"}= 3600;
+	$self->{"ttl"}= $datarrset->[0]->ttl;
     }
 
     $self->{"typecovered"}=$datarrset->[0]->type;  #Sanity checks elsewhere
@@ -310,10 +318,11 @@ sub create {
 
 
     my  $labels=$datarrset->[0]->name;
-    $labels =~ s/\.^//;  # remove trailing dot.
-    my @labels= split /\./ , $labels;
+    $labels =~ s/\.$//;  # remove trailing dot.
+    $labels =~ s/^\*\.//;  # remove initial asterisk label
+    my @labels= split /\./ , $labels;    # / emacs font-lock-mode	
     $self->{"labels"}= scalar(@labels);
-	
+
 
     # All the TTLs need to be the same in the data RRset.
     if ( @{$datarrset}>1 ){
@@ -333,18 +342,23 @@ sub create {
     # more ways to do things)
 
     bless $self, $class;
-
+    
     my $sigdata=$self->_CreateSigData($datarrset);
 
-
-
-
     my $signature;
     
     #
     # Enjoy the crypto
-    if ($self->algorithm == 1 || $self->algorithm == 5) {  #RSA
-	if (! ($Private->algorithm == 1 || $Private->algorithm == 5 )) {
+    if ($self->algorithm == 1 
+	|| $self->algorithm == 5
+	|| $self->algorithm == 7
+	|| $self->algorithm == 8
+	|| $self->algorithm == 10) {  #RSA
+	if (! ($Private->algorithm == 1 
+	       || $Private->algorithm == 5 
+	       || $Private->algorithm == 7 
+	       || $Private->algorithm == 8 
+	       || $Private->algorithm == 10  )) {
 	    die "Private key mismatch, not RSAMD5 or RSASHA.";
 	    
 	}
@@ -355,6 +369,10 @@ sub create {
 	    $rsa_priv->use_pkcs1_oaep_padding;
 	    if ($self->algorithm == 1) {
 		$rsa_priv->use_md5_hash;
+	    } elsif ($self->algorithm == 8) {
+    	$rsa_priv->use_sha256_hash;
+	    } elsif ($self->algorithm == 10) {
+    	$rsa_priv->use_sha512_hash;
 	    } else {
 		$rsa_priv->use_sha1_hash;
 	    }
@@ -368,7 +386,7 @@ sub create {
 
 	print "\n SIGNED" if $debug ;
 	
-    }elsif ($self->algorithm == 3){  #DSA
+    }elsif ($self->algorithm == 3 || $self->algorithm == 6  ){  #DSA
 	$self->{"private_key"}=$Private->privatekey;
 	my $private_dsa=$Private->privatekey;
 
@@ -414,7 +432,7 @@ sub create {
 
 
 sub verify {
-    my ($self, $dataref, $keyrr) = @_;
+    my ($self, $dataref, $keyrrref ) = @_;
 
     # Reminder...
 
@@ -423,13 +441,68 @@ sub verify {
     # object in the array...  @{$dataref} is length of the array when
     # called in numerical context
 
+    # $keyref is eiter a reference to an array of keys or a a key object.
+
     # if $dataref is not a reference it contains a string with data to be 
     # verified using SIG0
     
     my $sigzero_verify=0;
     my $packet_verify=0;
     my $rrarray_verify=0;
+
+    my $keyrr; # This will be used to store the key to which we want to 
+               # verify.
    
+
+    print "Second argument is of class".ref($keyrrref)."\n" if $debug;;
+    if (ref($keyrrref) eq "ARRAY"){
+	#  We will recurse for each key that matches algorithm and key-id 
+	#  we return when there is a succesful verification.
+        #  If not we'll continue so that we even survive key-id collission.
+	#  The downside of this is that the error string only matches the
+	#  last error.
+	my @keyarray=@{$keyrrref};
+	my $errorstring="";
+	my $i=0;
+	print "Itterating over " . @keyarray ." keys \n" if $debug;
+      KEYRR: foreach my $keyrr (@keyarray) {
+	  $i++;
+	  unless ($keyrr->algorithm == $self->algorithm){
+	      print "key $i: algorithm does not match\n" if $debug;
+	      $errorstring.="key $i: algorithm does not match ";
+	      next KEYRR;
+	  }
+	  unless ($keyrr->keytag == $self->keytag){
+	      print "key $i: keytag does not match (".$keyrr->keytag." ".$self->keytag.")\n" if $debug;
+	      $errorstring.="key $i: keytag does not match ";
+	      next KEYRR ;
+	      
+	  }
+	  my $result=$self->verify($dataref,$keyrr);
+	  print "key $i:".$self->vrfyerrstr if $debug;
+	  $errorstring.="key $i:".$self->vrfyerrstr." ";
+	  next KEYRR unless $result;
+	  $self->{"vrfyerrstr"}="No Error";
+	  return $result;
+      }
+	$self->{"vrfyerrstr"}=$errorstring;	  
+	return (0);
+    }elsif(ref($keyrrref) eq 'Net::DNS::RR::DNSKEY' ||
+	   ref($keyrrref) eq 'Net::DNS::RR::KEY' # we are liberal...
+	){
+	# substitute and continue processing after this conditional
+	$keyrr=$keyrrref;
+	print "Validating using key with keytag:".$keyrr->keytag." \n" if $debug;
+
+    }else{
+	# Error condition
+	$self->{"vrfyerrstr"} = "You are trying to pass ".ref($keyrrref) ." data for a key";
+	return (0);
+    }
+
+
+
+
     print "Verifying data of class:".  ref( $dataref) . "\n" if $debug;
     $sigzero_verify=1 unless (ref($dataref));
     if (! $sigzero_verify ){
@@ -450,6 +523,8 @@ sub verify {
 	}
     }
 
+
+
     $self->{"vrfyerrstr"}="---- Unknown Error Condition ------";
     print "\n ------------------------------- RRSIG DEBUG  -----------------\n"  if $debug;
     print "Reference: ".ref($dataref) if $debug;;
@@ -515,17 +590,25 @@ sub verify {
     if ( $self->algorithm == 1 ){    #Verifying for RSA
 	$verified=$self->_verifyRSA($sigdata,$signature,$keyrr,0) || return 0;
     }     
-    elsif ( $self->algorithm == 3 )  # Verifying for DSA
+    elsif ( $self->algorithm == 3 ||  $self->algorithm == 6 )  # Verifying for DSA
     {
 	 $verified=$self->_verifyDSA($sigdata,$signature,$keyrr) || return 0;
     }
-    elsif ( $self->algorithm == 5 )  # Verifying for RSASHA1
+    elsif ( $self->algorithm == 5 ||  $self->algorithm == 7 )  # Verifying for RSASHA1
     {
 	$verified=$self->_verifyRSA($sigdata,$signature,$keyrr,1) || return 0;
     }
+    elsif ( $self->algorithm == 8 )  # Verifying for RSASHA256
+    {
+	$verified=$self->_verifyRSA($sigdata,$signature,$keyrr,256) || return 0;
+    }
+    elsif ( $self->algorithm == 10 )  # Verifying for RSASHA512
+    {
+	$verified=$self->_verifyRSA($sigdata,$signature,$keyrr,512) || return 0;
+    }
     else                                  # Verifying other algorithms
     { 
-	$self->{"vrfyerrstr"}= "Algoritm ". $self->algorithm . " has not yet been implemented";
+	$self->{"vrfyerrstr"}= "Algorithm ". $self->algorithm . " has not yet been implemented";
 	return 0;
     }	
     
@@ -740,8 +823,12 @@ sub _verifyRSA {
 
     die "Could not load public key" unless $rsa_pub;
     $rsa_pub->use_pkcs1_oaep_padding;
-    if ($isSHA) {
+    if ($isSHA == 1) {
 	$rsa_pub->use_sha1_hash;
+    } elsif ($isSHA == 256) {
+	$rsa_pub->use_sha256_hash;
+    } elsif ($isSHA == 512) {
+	$rsa_pub->use_sha512_hash;
     } else {
 	$rsa_pub->use_md5_hash;
     }
@@ -795,9 +882,15 @@ sub _CreateSigData {
     my $sigdata;
     # construction of message 
 
+    # A hack to make a local copy of $rawdata in case we need to
+    # modify an RR name for wildcard validation
+    for (my $i=0; $i<@{$rawdata}; $i++){
+	$rawdata->[$i] = Net::DNS::RR->new($rawdata->[$i]->string);
+    }
 
+    print "_CreatSigData\n" if $debug;
     my $rdatawithoutsig=$self->rr_rdata_without_sigbin;
-    print "\n\nstrip:\t\t",  unpack("H*", $rdatawithoutsig) if $debug;
+    print "raw RRsig:\t",  unpack("H*", $rdatawithoutsig) if $debug;
     $sigdata= $rdatawithoutsig;
 
 
@@ -806,6 +899,16 @@ sub _CreateSigData {
 	if (@{$rawdata}>1) {
 	    my @canonicaldataarray;
 	    for (my $i=0; $i<@{$rawdata}; $i++){
+
+	        # check for wildcards
+	        my @components = Net::DNS::name2labels($rawdata->[$i]->{"name"});
+	        if (int(@components) > $self->{"labels"}) {
+		    print "\n fabricating wildcard RR name: " if $debug;
+		    shift @components while int(@components) > $self->{"labels"};
+		    $rawdata->[$i]->name(join('.', map(Net::DNS::wire2presentation($_),('*', @components))));
+		    print  $rawdata->[$i]->{"name"} . "\n" if $debug;
+	        }
+
 		if ($debug){
 		    print "Setting TTL to from ". $rawdata->[$i]->{"ttl"} . " to " .
 			$self->orgttl . "\n" 
@@ -814,7 +917,7 @@ sub _CreateSigData {
 		$rawdata->[$i]->{"ttl"}=$self->orgttl;
 		# Some error checking is done to. A RRset is defined by 
 		# Same label,class,qtype
-		if ($rawdata->[$i]->name ne $rawdata->[0]->name){
+		if (lc($rawdata->[$i]->name) ne lc($rawdata->[0]->name)){
 		    print "\nError:\n";
 		    for  (my $j=0; $j<@{$rawdata}; $j++){
 			print "\n";
@@ -880,10 +983,21 @@ sub _CreateSigData {
 		    $self->orgttl . "\n" if 
 			( $rawdata->[0]->{"ttl"}!=$self->orgttl );
 	    }
-	    print "\nRDATA: \t" .$rawdata->[0]->_canonicalRdata ."\t" .
-		unpack("H*",$rawdata->[0]->_canonicalRdata) ."\n" if $debug;
+	    print "\nRDATA:\t\t" .$rawdata->[0]->_canonicalRdata ."\n-----:\t\t" .
+	      unpack("H*",$rawdata->[0]->_canonicalRdata) ."\n" if $debug;
 	    
 	    $rawdata->[0]->{"ttl"}=$self->orgttl;	    
+
+	    #
+	    # check for wildcards
+	    #
+	    my @components = Net::DNS::name2labels($rawdata->[0]->{"name"});
+	    if (int(@components) > $self->{"labels"}) {
+		print "\n fabricating wildcard RR name:" if $debug;
+		shift @components while int(@components) > $self->{"labels"};
+		$rawdata->[0]->name(join('.', map(Net::DNS::wire2presentation($_),('*', @components))));
+		print  $rawdata->[0]->{"name"} . "\n" if $debug;
+	    }
 	    $sigdata .= $rawdata->[0]->_canonicaldata;
 	    
 	}
@@ -947,6 +1061,14 @@ sub siginceptation {
 }
 
 
+
+sub _normalize_dnames {
+	my $self=shift;
+	$self->_normalize_ownername();
+	$self->{'signame'}=lc(Net::DNS::stripdot($self->{'signame'})) if defined $self->{'signame'};
+}
+
+
 1;
 
 
@@ -984,7 +1106,7 @@ Create a signature over a RR set.
 					$keypath);
     my $sigrr= create Net::DNS::RR::RRSIG(\@datarrset,
 					$keypath,
-					\%arguments);
+					%arguments);
     $sigrr->print;
 
 
@@ -1034,8 +1156,8 @@ Notes:
   create method uses the filename as generated by dnssec-keygen to
   determine the keyowner, algorithm and the keyid (keytag).
 
-- Only RSA signatures (algorithm 1) and DSA signatures (algorithm 3)
-  have been implemented.
+- Only RSA signatures (algorithm 1,5 and 7) and DSA signatures 
+  (algorithm 3, and 6) have been implemented.
 
 
 
@@ -1087,7 +1209,7 @@ Read "KeyID Bug in bind." below.
 
     print "signame =", $rr->signame, "\n"
 
-Returns the name of the public KEY RRs  this sig was made with.
+Returns the name of the public DNSKEY RRs  this sig was made with.
 
 =head2 sig
 
@@ -1099,6 +1221,8 @@ Returns the base64 representation of the signature.
 =head2 verify and vrfyerrstr
 
     $sigrr->verify($data, $keyrr) || croak $sigrr->vrfyerrstr;
+    $sigrr->verify($data, [$keyrr, $keyrr2, $keyrr3]) || 
+                  croak $sigrr->vrfyerrstr;
 
 
 If $data contains a reference to an array of RR objects then them
@@ -1106,11 +1230,14 @@ method verifies the RRset against the signature contained in the
 $sigrr object itself using the public key in $keyrr.  Because of the
 KeyID bug in bind (see below) a check on keyid is not performed.
 
-
 If $data contains a reference to a Net::DNS::Packet and if $sig->type
 equals zero a a sig0 verification is performed. Note that the
 signature needs to be 'popped' from the packet before verifying.
 
+The second argument can either be a Net::DNS::RR::KEYRR object or a
+reference to an array of such objects. Verification will return
+successful as soon as one of the keys in the array leads to positive
+validation.
 
 Returns 0 on error and sets $sig->vrfyerrstr
 
@@ -1151,6 +1278,7 @@ requests.
 =head1 COPYRIGHT
 
 Copyright (c) 2001 - 2005  RIPE NCC.  Author Olaf M. Kolkman 
+Copyright (c) 2007 - 2008  NLnet Labs.  Author Olaf M. Kolkman 
 <olaf@net-dns.org>
 
 All Rights Reserved
